import 'dart:html' as html;
import 'dart:html';
import 'dart:svg' as svg;
import 'package:node_engine/src/nodes/custom/node_custom.dart';
import 'package:node_engine/src/nodes/custom/node_getter.dart';
import 'package:node_engine/src/nodes/custom/node_setter.dart';
import 'package:node_engine/src/nodes/math/node_substract.dart';
import 'package:node_engine/src/nodes/node.dart';
import 'package:node_engine/src/node_connection.dart';
import 'package:node_engine/src/node_utils.dart';
import 'node_field.dart';
import 'package:vector_math/vector_math.dart';

class NodeEditor{

  static NodeEditor _editor;
  static NodeEditor get editor => _editor;

  NodeField currentOutput;
  html.Element parentContainer;
  html.DivElement container;
  svg.SvgElement svgElement;

  List<NodeItem> nodes = new List<NodeItem>();

  factory NodeEditor.init({html.Element parent}){
    if(_editor == null){
      _editor = new NodeEditor._(parent:parent);
    }
    return _editor;
  }

  NodeEditor._({html.Element parent}){


    addCssStyleSheet();

    parentContainer = parent ?? html.document.body;

    container = new html.DivElement()..id = "node_engine_container";
    parentContainer.children.add(container);

    svgElement = new svg.SvgElement.tag("svg");
    container.children.add(svgElement);

    html.window.onClick.listen((e) {
      if (currentOutput != null) {
        currentOutput.tempConnection.deletePath();
        if (currentOutput.toNode != null) {
          //Todo (jpu) : ?? never pass here ??
//          currentOutput.toNode.detachOutput(currentOutput);
        }
        currentOutput = null;
      }
    });

    html.window.onMouseMove.listen((html.MouseEvent e) {
      if (currentOutput != null) {
        if(currentOutput.tempConnection == null){
          currentOutput.tempConnection = new Connection();
        }
        html.Point<int> mousePosition = new html.Point<int>(e.client.x, e.client.y);
        Point<int> toPoint = mousePosition - new Point<int>(parentContainer.documentOffset.x, parentContainer.documentOffset.y);

        currentOutput.drawTemporaryPath(toPoint);
      }
    });

    html.Element button = new html.ButtonElement()
      ..id = "buttonEvaluate"
      ..text = 'Evaluate'
      ..onClick.listen((html.MouseEvent event){
        evaluate();
        update();
      });
    container.children.add(button);

    _render();
  }

  void _render({num time: 0.0}) {

    try {
      update();
      evaluate();
    } catch (ex) {
      print("Error From NodeEditor :: _render method: $ex");
    }

    window.requestAnimationFrame((num time) {
      this._render(time: time);
    });
  }

  void addCssStyleSheet() {
    html.LinkElement linkElementCssStyle = new html.LinkElement()
      ..rel = 'stylesheet'
      ..type = 'text/css'
      ..media = 'screen'
      ..href = 'packages/node_engine/css/node_engine.css';
    html.document.head.append(linkElementCssStyle);
    html.CssStyleSheet currentSheet = linkElementCssStyle.sheet;currentSheet;//cheat unused Variable;// Todo (jpu) :is this line necessary ?
  }

  static Offset getFullOffset(html.Element element) {
    Vector3 transform = getCssTransform(element);

    Offset offset = new Offset()
      ..top = element.offsetTop + transform.y.toInt()
      ..left = element.offsetLeft + transform.x.toInt();

    if (element.offsetParent != null && element.parent.id != NodeEditor.editor.container.id) {
      Offset parentOffset = getFullOffset(element.offsetParent);
      offset.top += parentOffset.top;
      offset.left += parentOffset.left;
    }

    return offset;
  }

  static Vector3 getCssTransform(html.Element element) {
    Vector3 result = new Vector3.zero();

    RegExp regExp = new RegExp(r'translate3d\((.+)px, (.+)px, (.+)px\)');
    var matches = regExp.allMatches(element.style.transform);
    if(matches.length > 0) {
      var match = matches.elementAt(0); // => extract the first (and only) match
      result.setValues(
          double.parse(match.group(1)),
          double.parse(match.group(2)),
          double.parse(match.group(3)));
    }

    return result;
  }

  void evaluate() {
    nodes.where((n)=> n.isLauncher == true).forEach((n) => n.evaluate());
  }

  void update(){
    nodes.forEach((n)=> n.updatePosition());
  }

  void createNodeByType(NodeType nodeType){
    switch(nodeType){
      case NodeType.custom:
          new NodeCustom('custom');
        break;
      case NodeType.add:
          new NodeAdd();
        break;
      case NodeType.substract:
          new NodeSubstract();
        break;
      case NodeType.divide:
          new NodeDivide();
        break;
      case NodeType.multiply:
          new NodeMultiply();
        break;
      case NodeType.log:
          new NodeLog();
        break;
      case NodeType.int:
          new NodeInt(0);
        break;
      case NodeType.getter:
          new NodeGetter(null);
        break;
      case NodeType.setter:
          new NodeSetter(null);
        break;
    }
  }
}