import 'dart:html' hide Node;
import 'package:node_engine/src/node_editor.dart';
import 'package:node_engine/src/nodes/custom/node_custom.dart';
import 'package:node_engine/src/nodes/custom/node_getter.dart';
import 'package:node_engine/src/nodes/custom/node_setter.dart';
import 'package:node_engine/src/nodes/node.dart';

///Exemple de code, inspiration
///https://codepen.io/xgundam05/pen/KjqJn

///dnd : drag and drop dart library
///http://code.makery.ch/library/dart-drag-and-drop/

void main() {
//  simpleExemple();
//  simpleExemple2();
  fullExemple();
//  elementConnectionExemple();
}

void simpleExemple() {

  new NodeEditor.init();

  // Node 1
  NodeItem node01 = new NodeCustom('Node 01');
  node01.addOutputField<int>('output 0', 10);
  node01.addOutputField<int>('output 1', 20);
  node01.position = new Point<int>(20, 20);

  // Node 2
  NodeItem node02 = new NodeCustom('Node 02');
  node02.addInputField<int>('input 0', 0);
  node02.addInputField<int>('input 1', 0);
  node02.addInputField<int>('input 2', 0);
  node02.position = new Point<int>(400, 20);

  node01.outputFields[0].connectTo(node02.inputFields[0]);
}

void simpleExemple2() {

  new NodeEditor.init();

  NodeItem node01 = new NodeCustom('Node 01');
  node01.addOutputField('01 output 01', 5);
  node01.position = new Point<int>(20, 40);

  NodeItem node02 = new NodeCustom('Node 02');
  node02.addOutputField('02 output 01', 9);
  node02.addInputField('02 input 01', 3);
  node02.position = new Point<int>(300, 40);
  node02.evaluationFunction = (NodeItem node) {
    node.outputFields[0].value = node.inputFields[0].value;
  };

  NodeItem node03 = new NodeCustom('Node 03');
  node03.isLauncher = true;
  node03.addInputField('03 input 01', 7);
  node03.position = new Point<int>(600, 40);

  node01.outputFields.first.connectTo(node02.inputFields[0]);
  node02.outputFields.first.connectTo(node03.inputFields[0]);
}

void fullExemple() {

  NodeEditor editor = new NodeEditor.init();

  NodeItem nodeInt01 = new NodeInt(2)..position = new Point<int>(20, 160);
  NodeItem nodeInt02 = new NodeInt(3)..position = new Point<int>(20, 240);
  NodeItem nodeInt03 = new NodeInt(4)..position = new Point<int>(420, 360);
  NodeItem nodeDivide = new NodeDivide()..position = new Point<int>(250, 120);
  NodeItem nodeAdd = new NodeAdd()..position = new Point<int>(480, 200);
  NodeItem nodeMultiply = new NodeMultiply()..position = new Point<int>(690, 280);
  NodeItem nodeLog = new NodeLog()..position = new Point<int>(900, 300);

  // Connect Nodes
  nodeInt01.outputFields[0].connectTo(nodeDivide.inputFields[0]);
  nodeInt02.outputFields[0].connectTo(nodeAdd.inputFields[1]);
  nodeInt02.outputFields[0].connectTo(nodeDivide.inputFields[1]);
  nodeDivide.outputFields[0].connectTo(nodeAdd.inputFields[0]);
  nodeAdd.outputFields[0].connectTo(nodeMultiply.inputFields[0]);
  nodeInt03.outputFields[0].connectTo(nodeMultiply.inputFields[1]);
  nodeMultiply.outputFields[0].connectTo(nodeLog.inputFields[0]);

  editor.update();
}

void elementConnectionExemple() {
  new NodeEditor.init();

//  NodeItem nodeInt01 = new NodeInt(100)..position = new Point<int>(200, 500);
//  NodeItem nodeInt02 = new NodeInt(10)..position = new Point<int>(200, 100);
//  NodeItem nodeInt03 = new NodeInt(5)..position = new Point<int>(200, 150);
//  NodeItem nodeMultiply = new NodeMultiply()..position = new Point<int>(500, 200);
//
//  nodeInt02.outputFields[0].connectTo(nodeMultiply.inputFields[0]);
//  nodeInt03.outputFields[0].connectTo(nodeMultiply.inputFields[1]);

//  nodeMultiply.evaluate();

  RectangleItem item01 = new RectangleItem()
    ..x = 50
    ..y = 100;

  RectangleItem item02 = new RectangleItem()
    ..x = 0
    ..color = '#00ff00';

  ///  node can be used directly to set values, but no correct graph is drawn
  //  item.y = nodeInt01.outputFields[0].value;
  //  item.x = nodeMultiply.outputFields[0].value;

  /// Or add a custom node with a custom function and custom object, and have a correct graph
//  NodeItem customNode = new NodeCustom('RectangleItem')
//  ..isLauncher = true
//  ..position = new Point<int>(1000, 350)
//  ..addInputField<int>('x', 0)
//  ..addInputField<int>('y', 0)
//  ..objectContent = item01
//  ..evaluationFunction = (NodeItem node) {
//    Positionable posObject = (node as NodeCustom).objectContent as Positionable;
//    posObject
//        ..x = node.inputFields[0].value
//        ..y = node.inputFields[1].value;
//  };
//
//  nodeMultiply.outputFields[0].connectTo(customNode.inputFields[0]);
//  nodeInt01.outputFields[0].connectTo(customNode.inputFields[1]);


  ///Using getter and setters

  //Getter >> inputField
  //a
//  PropertyGetter getterItem01x = () => item01.x;
//  NodeGetter nodeGetter = new NodeGetter<int>(getterItem01x);
  //b
//  NodeGetter nodeGetter = new NodeGetter<int>(() => item01.x);
  //c
  NodeGetter nodeGetter = new NodeGetter<int>()
    ..getter = () => item01.x;

  //outputField >> Setter
  //a
//  PropertySetter setterItem02x = (dynamic v) => item02.x = v;
//  NodeSetter nodeSetter = new NodeSetter<int>(setterItem02x)
  //b
//   NodeSetter nodeSetter = new NodeSetter<int>((dynamic v) => item02.x = v);
  //C
   NodeSetter nodeSetter = new NodeSetter<int>()
  ..setter = (dynamic v) => item02.x = v;

  nodeSetter
    ..position = new Point<int>(500, 100)
    ..isLauncher = true;

  nodeGetter.outputFields[0].connectTo(nodeSetter.inputFields[0]);

  nodeSetter.evaluate();
}
class RectangleItem{

  Element element;

  int get x => int.parse(element.style.left.replaceAll('px', ''));
  int get y => int.parse(element.style.top.replaceAll('px', ''));

  set x (int value) {
    element.style.left = '${value}px';
  }
  set y (int value) {
    element.style.top = '${value}px';
  }

  set color(String hexColor) => element.style.backgroundColor = hexColor;

  RectangleItem(){
    element = new DivElement();
    element.classes.add('testItem');
    document.body.children.add(element);

    LinkElement linkElementCssStyle = new LinkElement()
      ..rel = 'stylesheet'
      ..type = 'text/css'
      ..media = 'screen'
      ..href = 'main.css';
    document.head.append(linkElementCssStyle);
    CssStyleSheet currentSheet = linkElementCssStyle.sheet;currentSheet;//cheat unused Variable;//

  }
}

// Todo (jpu) : use index setter and automatic connect
//  customNode.inputFields[0] = nodeMultiply.outputFields[0];
//  customNode.inputFields[1] = nodeInt01.outputFields[0];